@@ -1,10 +1,3 @@
-stack.trace <- function(expr) {
-  tryCatch(expr,Exception = function(e){
-    print(e)
-    cat(paste(unlist(lapply(.jevalArray(e$jobj$getStackTrace()),function(x) { x$toString() })),collapse="\n"),"\n",sep="")
-  })
-}
-
 bamboo.priorMSA <- function(countsMatrix,alpha=c(1,1,1,1)) {
   if ( length(alpha) != 4 ) stop("alpha must have length 4.")
   alpha <- c(alpha,0,0,0)
@@ -15,13 +8,14 @@ bamboo.priorMSA <- function(countsMatrix,alpha=c(1,1,1,1)) {
   } else {
     if ( ncol(y) == 4 ) y <- cbind(y,0,0,0)
     storage.mode(y) <- "integer"
-    .jcall(.jnew("org/ddahl/bamboo/PriorMSA$"),"Lorg/ddahl/bamboo/PriorMSA;","apply",.jarray(y,dispatch=TRUE),as.double(alpha))
+    ##rJava## .jcall(.jnew("org/ddahl/bamboo/PriorMSA$"),"Lorg/ddahl/bamboo/PriorMSA;","apply",.jarray(y,dispatch=TRUE),as.double(alpha))
+    s$.org.ddahl.bamboo.PriorMSA$apply(y,alpha)
   }
 }
 
 bamboo.priorNonInfo <- function() {
-  x <- .jcall(.jnew("org/ddahl/bamboo/PriorNoHE$"),"Lorg/ddahl/bamboo/PriorNoHE;","apply")
-  x
+  ##rJava## .jcall(.jnew("org/ddahl/bamboo/PriorNoHE$"),"Lorg/ddahl/bamboo/PriorNoHE;","apply")
+  s$.org.ddahl.bamboo.PriorNoHE$apply()
 }
 
 deduce.model <- function(secondary) {
@@ -31,43 +25,52 @@ deduce.model <- function(secondary) {
 }
 
 bamboo.priorMM <- function(secondary,order=(1:9)[7],countsFile=NA,force=FALSE,warn=TRUE) {
-  gc()
   model <- deduce.model(secondary)
   priorStr <- paste("PriorMM",order,sep="")
-  priorClass <- .jnew(paste("org/ddahl/bamboo/",priorStr,"$",sep=""))
-  priorSignature <- paste("Lorg/ddahl/bamboo/",priorStr,";",sep="")
+  ##rJava## priorClass <- .jnew(paste("org/ddahl/bamboo/",priorStr,"$",sep=""))
+  priorClass <- paste("org.ddahl.bamboo.",priorStr,sep="")
   if ( is.na(countsFile) ) countsFile <- paste(model,priorStr,sep=.Platform$file.sep)
-  bag <- .jcall(.jnew("org/ddahl/bamboo/BagOfPMFs$"),"Lorg/ddahl/bamboo/BagOfPMFs;","apply",FALSE)
-  prior <- .jcall(priorClass,priorSignature,"apply",bag)
+  ##rJava## bag <- .jcall(.jnew("org/ddahl/bamboo/BagOfPMFs$"),"Lorg/ddahl/bamboo/BagOfPMFs;","apply",FALSE)
+  bag <- s$.org.ddahl.bamboo.BagOfPMFs$apply(I(FALSE))
+  ##rJava## priorSignature <- paste("Lorg/ddahl/bamboo/",priorStr,";",sep="")
+  ##rJava## prior <- .jcall(priorClass,priorSignature,"apply",bag)
+  prior <- s$do(priorClass)$apply(bag)
   if ( force || ! file.exists(countsFile) ) {
-    .jcall(prior,"V","count",secondary,countsFile)
+    ##rJava## .jcall(prior,"V","count",secondary,countsFile)
+    prior$count(secondary,I(countsFile))
   } else {
     if ( warn ) warning("Using existing counts file, so ignoring secondary argument. Suppress this warning with warn=FALSE.")
   }
-  bag <- .jcall(.jnew("org/ddahl/bamboo/BagOfPMFs$"),"Lorg/ddahl/bamboo/BagOfPMFs;","apply",countsFile,FALSE)
-  x <- .jcall(priorClass,priorSignature,"apply",bag)
+  ##rJava## bag <- .jcall(.jnew("org/ddahl/bamboo/BagOfPMFs$"),"Lorg/ddahl/bamboo/BagOfPMFs;","apply",countsFile,FALSE)
+  bag <- s$.org.ddahl.bamboo.BagOfPMFs$apply(I(countsFile),I(FALSE))
+  ##rJava## x <- .jcall(priorClass,priorSignature,"apply",bag)
+  x <- s$do(priorClass)$apply(bag)
   x
 }
 
 bamboo.likelihood.engine <- function(string,aa=NA) {
   x <- if ( ! is.na(aa) ) {
     if ( ! all(grepl("^[ARNDCEQGHILKMFPSTWYV]+$",aa)) ) stop("Amino acid sequence must use only A,R,N,D,C,E,Q,G,H,I,L,K,M,F,P,S,T,W,Y,V.")
-    aaObj <- .jcall(.jnew("org/ddahl/bamboo/AminoAcidSequence$"),"Lorg/ddahl/bamboo/AminoAcidSequence;","apply",aa)
-    .jcall(.jnew("org/ddahl/bamboo/Likelihood$"),"Lorg/ddahl/bamboo/Likelihood;","apply",string,aaObj)
+    ##rJava## aaObj <- .jcall(.jnew("org/ddahl/bamboo/AminoAcidSequence$"),"Lorg/ddahl/bamboo/AminoAcidSequence;","apply",aa)
+    aaObj <- s$.org.ddahl.bamboo.AminoAcidSequence$apply(I(aa))
+    ##rJava## .jcall(.jnew("org/ddahl/bamboo/Likelihood$"),"Lorg/ddahl/bamboo/Likelihood;","apply",string,aaObj)
+    s$.org.ddahl.bamboo.Likelihood$apply(I(string),aaObj)
   } else {
-    factory <- .jcall("org/ddahl/bamboo/Likelihood","Lscala/Function1;","factory",string)
-    aasd <- .jnew("org/ddahl/bamboo/AminoAcidSequence$")
+    ##rJava## factory <- .jcall("org/ddahl/bamboo/Likelihood","Lscala/Function1;","factory",string)
+    factory <- s$.org.ddahl.bamboo.Likelihood$factory(I(string))
+    ##rJava## aasd <- .jnew("org/ddahl/bamboo/AminoAcidSequence$")
     function(aa) {
       if ( ! all(grepl("^[ARNDCEQGHILKMFPSTWYV]+$",aa)) ) stop("Amino acid sequence must use only A,R,N,D,C,E,Q,G,H,I,L,K,M,F,P,S,T,W,Y,V.")
-      aaObj <- .jcall(aasd,"Lorg/ddahl/bamboo/AminoAcidSequence;","apply",aa)
-      .jcall(factory,"Lorg/ddahl/bamboo/Likelihood;","apply",aaObj)
+      ##rJava## aaObj <- .jcall(aasd,"Lorg/ddahl/bamboo/AminoAcidSequence;","apply",aa)
+      aaObj <- s$.org.ddahl.bamboo.AminoAcidSequence$apply(I(aa))
+      ##rJava## .jcall(factory,"Lorg/ddahl/bamboo/Likelihood;","apply",aaObj)
+      factory$apply(aaObj)
     }
   }
   x
 }
 
 bamboo.likelihood <- function(primary,secondary,countsDirectory="HETC",force=FALSE,warn=TRUE) {
-  gc()
   if ( ! all(grepl("^[ARNDCEQGHILKMFPSTWYV]+$",primary)) ) stop("Amino acid sequence must use only A,R,N,D,C,E,Q,G,H,I,L,K,M,F,P,S,T,W,Y,V.")
   model <- deduce.model(secondary)
   if ( is.na(countsDirectory) ) countsDirectory <- model
@@ -79,7 +82,8 @@ bamboo.likelihood <- function(primary,secondary,countsDirectory="HETC",force=FAL
     str2 <- gsub("\\*",countsDirectory,"H:*/H:*/H_r;E:*/E;T:*/T;C:*/C","\\*")
   }
   if ( force || ! file.exists(countsDirectory) ) {
-    .jcall(bamboo.likelihood.engine(str1,"A"),"V","count",primary,secondary,countsDirectory)
+    ##rJava## .jcall(bamboo.likelihood.engine(str1,"A"),"V","count",primary,secondary,countsDirectory)
+    bamboo.likelihood.engine(str1,"A")$count(primary,secondary,I(countsDirectory))
   } else {
     if ( warn ) warning("Using existing counts directory, so ignoring primary and secondary arguments. Suppress this warning with warn=FALSE.")
   }
@@ -87,8 +91,8 @@ bamboo.likelihood <- function(primary,secondary,countsDirectory="HETC",force=FAL
 }
 
 bamboo.estimate <- function(likelihood,prior,nSamples,dropFirst,initialState=NULL,doLeastSquaresEstimation=FALSE,dumpStates=FALSE) {
-  gc()
-  n <- .jcall(.jcall(likelihood,"Lorg/ddahl/bamboo/AminoAcidSequence;","aa"),"I","nPositions")
+  ##rJava## n <- .jcall(.jcall(likelihood,"Lorg/ddahl/bamboo/AminoAcidSequence;","aa"),"I","nPositions")
+  n <- likelihood$aa()$nPositions()
   if ( is.null(initialState) ) {
     keyString <- paste(sort(strsplit(likelihood$blockLikelihoodMap()$keys()$mkString(),"")[[1]]),collapse="")
     if ( keyString == "KOS" ) {
@@ -101,19 +105,30 @@ bamboo.estimate <- function(likelihood,prior,nSamples,dropFirst,initialState=NUL
       initialState <- bamboo.sanitize(paste(initialState,collapse=""))
     } else stop("Unexpected model.")
   } else if ( nchar(initialState) != n ) stop("Length of initial state is not compatible with the likelihood.")
-  stateObj <- .jcall("org/ddahl/bamboo/Bamboo","Lorg/ddahl/bamboo/Bamboo;","apply",initialState)
-  prior <- .jcast(prior,"org/ddahl/bamboo/Prior")
-  result <- .jcall("org/ddahl/bamboo/Estimate","Lorg/ddahl/bamboo/MCMCResults;","apply",likelihood,prior,stateObj,as.integer(nSamples),as.integer(dropFirst),as.logical(doLeastSquaresEstimation),as.logical(dumpStates))
-  marginalProbabilities = .jevalArray(.jcall(result,"[[D","marginalProbabilities",evalArray=FALSE),simplify=TRUE)
-  colnames(marginalProbabilities) <- .jcall("org/ddahl/bamboo/package","[S","SSOrder")
+  ##rJava## stateObj <- .jcall("org/ddahl/bamboo/Bamboo","Lorg/ddahl/bamboo/Bamboo;","apply",initialState)
+  stateObj <- s$.org.ddahl.bamboo.Bamboo$apply(I(initialState))
+  ##rJava## prior <- .jcast(prior,"org/ddahl/bamboo/Prior")
+  ##rJava## result <- .jcall("org/ddahl/bamboo/Estimate","Lorg/ddahl/bamboo/MCMCResults;","apply",likelihood,prior,stateObj,as.integer(nSamples),as.integer(dropFirst),as.logical(doLeastSquaresEstimation),as.logical(dumpStates))
+  result <- s$.org.ddahl.bamboo.Estimate$apply(likelihood,prior,stateObj,I(as.integer(nSamples)),I(as.integer(dropFirst)),I(as.logical(doLeastSquaresEstimation)),I(as.logical(dumpStates)))
+  ##rJava## marginalProbabilities = .jevalArray(.jcall(result,"[[D","marginalProbabilities",evalArray=FALSE),simplify=TRUE)
+  marginalProbabilities = result$marginalProbabilities()
+  ##rJava## colnames(marginalProbabilities) <- .jcall("org/ddahl/bamboo/package","[S","SSOrder")
+  colnames(marginalProbabilities) <- s$.org.ddahl.bamboo$SSOrder()
   x <- list(
-    countTotal      = .jcall(result,"I","countTotal"),
-    countBad        = .jcall(result,"I","countBad"),
-    countAccepted   = .jcall(result,"I","countAccepted"),
-    mapState        = .jcall(.jcall("org/ddahl/bamboo/Bamboo","Lscala/collection/immutable/List;","toSequence",.jcall(result,"Lscala/collection/immutable/List;","mapState")),"S","mkString",""),
-    maxLogPosterior = .jcall(result,"D","maxLogPosterior"),
-    mpState         = .jcall(.jcall("org/ddahl/bamboo/Bamboo","Lscala/collection/immutable/List;","toSequence",.jcall(result,"Lscala/collection/immutable/List;","mpState")), "S","mkString",""),
-    lsState         = .jcall(.jcall("org/ddahl/bamboo/Bamboo","Lscala/collection/immutable/List;","toSequence",.jcall(result,"Lscala/collection/immutable/List;","lsState")), "S","mkString",""),
+    ##rJava## countTotal      = .jcall(result,"I","countTotal"),
+    countTotal      = result$countTotal(),
+    ##rJava## countBad        = .jcall(result,"I","countBad"),
+    countBad        = result$countBad(),
+    ##rJava## countAccepted   = .jcall(result,"I","countAccepted"),
+    countAccepted   = result$countAccepted(),
+    ##rJava## mapState        = .jcall(.jcall("org/ddahl/bamboo/Bamboo","Lscala/collection/immutable/List;","toSequence",.jcall(result,"Lscala/collection/immutable/List;","mapState")),"S","mkString",""),
+    mapState        = s$.org.ddahl.bamboo.Bamboo$toSequence(result$mapState())$mkString(I("")),
+    ##rJava## maxLogPosterior = .jcall(result,"D","maxLogPosterior"),
+    maxLogPosterior = result$maxLogPosterior(),
+    ##rJava## mpState         = .jcall(.jcall("org/ddahl/bamboo/Bamboo","Lscala/collection/immutable/List;","toSequence",.jcall(result,"Lscala/collection/immutable/List;","mpState")), "S","mkString",""),
+    mpState         = s$.org.ddahl.bamboo.Bamboo$toSequence(result$mpState())$mkString(I("")),
+    ##rJava## lsState         = .jcall(.jcall("org/ddahl/bamboo/Bamboo","Lscala/collection/immutable/List;","toSequence",.jcall(result,"Lscala/collection/immutable/List;","lsState")), "S","mkString",""),
+    lsState         = s$.org.ddahl.bamboo.Bamboo$toSequence(result$lsState())$mkString(I("")),
     marginalProbabilities = marginalProbabilities[,c("H","E","T","C")]
   )
   class(x) <- "bamboo.estimate"
@@ -126,8 +141,10 @@ plot.bamboo.estimate <- function(x,ss=NULL,...) {
   nHeaders <- length(ss)
   headerHeight <- 0.1
   n <- nrow(x$marginalProbabilities)
-  if ( length(dev.list()) == 0 ) options()$device(width=14,height=6)
-  else {
+  if ( length(dev.list()) == 0 ) {
+    func <- options()$device
+    if ( is.function(func) ) func(width=14,height=6)
+  } else {
     ds <- dev.size()
     ar <- ds[1]/ds[2]
     if ( ( ar < 2 ) || ( ar > 3 ) ) warning("For best results, the plotting device should have an aspect ratio about 14:6.  Closing the current device is recommended.")
@@ -174,39 +191,3 @@ plot.bamboo.estimate <- function(x,ss=NULL,...) {
 
 }
 
-bamboo <- function(likelihood,prior) {
-  targetState <- .jnull("org/ddahl/bamboo/Bamboo")
-  propose <- function(state) {
-    if ( class(state) == "bamboo.state" ) {
-      stateObj <- state$proposalObj
-    } else {
-      stateObj <- .jcall("org/ddahl/bamboo/Bamboo","Lorg/ddahl/bamboo/Bamboo;","apply",state)
-    }
-    map <- .jcall(stateObj,"Lscala/collection/immutable/Map;","propose$default$1")
-    result <- .jcall(stateObj,"Lscala/Tuple2;","propose",map,targetState)
-    proposal <- .jcast(.jcall(result,"Ljava/lang/Object;","_1"),"org/ddahl/bamboo/Bamboo")
-    noChange = .jcall(proposal,"Z","sameInstanceAs",stateObj)
-    if ( noChange ) {
-      if ( class(state) == "bamboo.state" ) {
-        proposalString <- state$proposalString
-      } else {
-        proposalString <- state
-      }
-      logLikelihood <- 0.0
-      logPrior <- 0.0
-      logRatioProposalProbability <- 0.0
-    } else {
-      proposalString <- .jcall(proposal,"S","asString")
-      bambooState <- .jcast(proposal,"org/ddahl/bamboo/Bamboo")
-      logLikelihood <- .jcall(likelihood,"D","apply",bambooState)
-      logPrior <- .jcall(prior,"D","apply",bambooState)
-      logRatioProposalProbability <- .jcall(.jcall(result,"Ljava/lang/Object;","_2"),"D","doubleValue")
-    }
-    result <- list(proposal=proposalString,proposalObj=proposal,logLikelihood=logLikelihood,logPrior=logPrior,logRatioProposalProbability=logRatioProposalProbability,noChange=noChange)
-    class(result) <- "bamboo.state"
-    result
-  }
-  class(propose) <- "bamboo.proposer"
-  propose
-}
-
